@@ -22,6 +22,12 @@
 #ifndef __GST_VP8_ENC_H__
 #define __GST_VP8_ENC_H__
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
+#ifdef HAVE_VP8_ENCODER
+
 #include <gst/gst.h>
 #include <gst/video/gstvideoencoder.h>
 
@@ -67,6 +73,8 @@ struct _GstVP8Enc
   gint n_ts_layer_id;
   /* Global two-pass options */
   gchar *multipass_cache_file;
+  gchar *multipass_cache_prefix;
+  guint multipass_cache_idx;
   GByteArray *first_pass_cache_content;
 
   /* Encode parameter */
@@ -96,7 +104,7 @@ struct _GstVP8Enc
 
   vpx_image_t image;
 
-  int n_frames;
+  GstClockTime last_pts;
   int keyframe_distance;
 
   GstVideoCodecState *input_state;
@@ -111,4 +119,6 @@ GType gst_vp8_enc_get_type (void);
 
 G_END_DECLS
 
+#endif
+
 #endif /* __GST_VP8_ENC_H__ */
